/*! 
  \file fileinfo.cpp
  \brief fileinfo dialog

  \author R. Enders

  $Revision: 1.4 $
  $Id: fileinfo.cpp,v 1.4 2006/01/06 12:17:59 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include fileinfo.cpp
*/

#include "config.h"
#include <assert.h>
#include <math.h>
#include <string>

#include <qtable.h>

#include ".ui/fileinfobase.h"
#include "fileinfo.h"
#include "pesmgr.h"

using namespace std;

Fileinfo::Fileinfo(QWidget* parent, const char* name,
			 bool modal, WFlags fl)
    : FileinfoDialog(parent, name, modal, fl)
{}

Fileinfo::~Fileinfo()
{

}

void Fileinfo::showdata(PesMgr& pesmgr)
{
    char buff[1024];

    int row = 0;
    FileInfoTable->setText(row,0,"File Name");
    FileInfoTable->setText(row,1,QString(pesmgr.getFileName().c_str()));

    ++row;
    FileInfoTable->setText(row,0,"File Size");
    sprintf(buff,"%llu bytes",pesmgr.getFsize());
    FileInfoTable->setText(row,1,buff);

    ++row;
    FileInfoTable->setText(row,0,"File Length");
    int s = pesmgr.getLastClipPoint().ms / 1000;
    sprintf(buff,"%d:%02d:%02d", s / 3600, (s / 60) % 60, s % 60);
    FileInfoTable->setText(row,1,buff);

    ++row;
    FileInfoTable->setText(row,0,"File Modtime");
    time_t ftm = pesmgr.getMtime();
    struct tm& l = *localtime(&ftm);
    sprintf(buff,"%d-%02d-%02d %02d:%02d",
	    l.tm_year+1900, l.tm_mon+1, l.tm_mday,
	    l.tm_hour, l.tm_min);
    FileInfoTable->setText(row,1,buff);

    ++row;
    FileInfoTable->setText(row,0,"Frame Rate");
    sprintf(buff,"%0.2f Hz",pesmgr.getFrameRate());
    FileInfoTable->setText(row,1,buff);

    ++row;
    FileInfoTable->setText(row,0,"Bit Rate");
    sprintf(buff,"%d bits/sec",pesmgr.getBitRate());
    FileInfoTable->setText(row,1,buff);

    ++row;
    FileInfoTable->setText(row,0,"Picture Width");
    sprintf(buff,"%d pixel",pesmgr.getPWidth());
    FileInfoTable->setText(row,1,buff);

    ++row;
    FileInfoTable->setText(row,0,"Picture Height");
    sprintf(buff,"%d pixel",pesmgr.getPHeight());
    FileInfoTable->setText(row,1,buff);

    FileInfoTable->adjustColumn(0);
    FileInfoTable->adjustColumn(1);
}
