/*! 
  \file gopdit.cpp
  \brief

  \author R. Enders

  $Revision: 1.17 $
  $Id: gopdit_qt.cpp,v 1.17 2006/06/04 15:58:01 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  mdify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include gopdit.cpp
*/

#include "config.h"
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <qfiledialog.h>
#include <qapplication.h>
#include <qslider.h>
#include <qtable.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qprogressdialog.h>
#include <qpushbutton.h>
#include <qsplitter.h>
#include <qstatusbar.h>
#include <qstring.h>
#include <qstringlist.h>
#include "qxvideo.h"

#include "parameter.h"
#include ".ui/gopditbase.h"
#include "pesmgr.h"
#include "mpeg2output.h"
#include "gopdit_qt.h"
#include "gopdithelp.h"
#include "editlist.h"
#include "fileinfo.h"
#include "parfiledialog.h"
#include "preferences.h"

#include "images/PauseAlpha.xpm"
#include "images/Play.xpm"
#include "images/Red.xpm"
#include "images/Yellow.xpm"
#include "images/Green.xpm"



#define OFF_T_FORMAT ".2f"

/*
#undef OFF_T_FORMAT
#if _FILE_OFFSET_BITS==64
# define OFF_T_FORMAT  "llu"
#else
# warning "Not compiling for large file (>2GB) support!"
# define OFF_T_FORMAT  "lu"
#endif
*/



#define PROGRESS_DIALOG

using std::vector;
using std::string;

const double gop_div  = 1000 * 1000;
//const int gop_div  = 8 * 1024;
//const int gop_div  = 1;
const int prog_div = 1024 * 1024;

using namespace std;

extern Parameter<bool> filter;
extern Parameter<bool> separate;

Parameter<bool> gop_inclusive("GopInclusive",true);
Parameter<bool> display_offset("DisplayOffset",false);
Parameter<bool> import_mpegs("ImportMpegs",false);

FileDialogParameter open_file("OpenFile",
			      "Mpeg (*.mpg *.mpg2);;All Files (*.*)"
			      ";;TiVo (experimental!) (*.ty)",
			      "Mpeg (*.mpg *.mpg2)",
			      "Open",
			      ".");

FileDialogParameter save_file("SaveFile",
			      "Mpeg (*.mpg *.mpg2);;Vob (*.vob)",
			      "Mpeg (*.mpg *.mpg2)",
			      "Save As",
			      ".");

Parameter<int> toggle_steps("ToggleZoomSteps",5);

Parameter<int> main_width("MainWidth",550);
Parameter<int> main_height("MainHeight",420);

Parameter<int> frame1_width("Frame1Width",336);
Parameter<int> frame2_width("Frame2Width",208);

// xine has it's own time scale - so we convert
Parameter<float> time_scale("TimeScale",1.0);
Parameter<bool> activate_time_scale("ActivateTimeScale",false);

Parameter<bool> mark_clips("MarkClips",false);
// mark red, if smaller than 35 secs
Parameter<unsigned> red_mark("RedClipMark",35000);
// mark yellow, if smaller than 5 mins
Parameter<unsigned> yellow_mark("YellowClipMark",300000);
// when true, erase clips after save
Parameter<bool> erase_clips("EraseClips",false);

// we probably would need a class for slider operations
// especially for scaling

GopditQt::GopditQt(QApplication& papp,
		       QWidget* parent, const char* name, WFlags fl)
    : GopditBase(parent, name, fl)
    , modified(false)
    , finfo(0)
    , help(0)
    , prefs(0)
    , app(papp)
    , prog_diag(0)
    , pause(PauseAlpha)
    , play(Play)    
{
    ui = this;

// initialize the clip table to our needs
// how much could be done by 'designer'?

    Clips->setDragEnabled(true);
    Clips->setRowMovingEnabled(false);
    Clips->setReadOnly(true);
//    Clips->setSelectionMode(QTable::MultiRow);

    for ( unsigned i = 0; i < 4; ++i ) 
    {
	Clips->adjustColumn(i);
    }

// initialize application geometry
    resize(main_width.getValue(), main_height.getValue());

    QValueList<int> width_list;
    width_list.push_back(frame1_width.getValue());
    width_list.push_back(frame2_width.getValue());
    Splitter->setSizes(width_list);
}

GopditQt::~GopditQt()
{
    if ( finfo != 0 ) 
    {
	delete finfo;
    }
    if ( help != 0 ) 
    {
	delete help;
    }
    if ( prefs != 0 ) 
    {
	delete prefs;
    }
}

// tests are missing, if the file is existent and readable
void GopditQt::fileOpen()
{
    processEvents();
    
    ParFileDialog filedialog(open_file, this, "Open", TRUE,
			     QFileDialog::ExistingFile);
    if ( filedialog.exec() == QDialog::Accepted )
    {
	processEvents();
	fileOpen(string(filedialog.selectedFile().ascii()));
    }
}

void GopditQt::fileOpen(string filename)
{
    if ( openFile(filename) )
    {
	QString fn = QString(filename.c_str());
	
	// the following should be a function
	// currently just for testing
	uint to = fn.findRev('.');
	uint from = fn.findRev('/');
	QString moviename = fn.mid(from+1,to-from-1);

	CurMovie->insertItem(moviename,pesmgrv.size()-1);
	CurMovie->setCurrentItem(pesmgrv.size()-1);

	// also initialize first gop!
	slider_begin = 0;
	slider_end   = getFsize();
	Zoom->setText("");

	MovieScreen->setPesMgr(pesmgr);
	pesmgr->play(1,true);
	movieSliderChanged(0);
    }
}




void GopditQt::fileClose()
{
    if ( pesmgr == 0 )
    {
	return;
    }

    // handle combobox and PesMgr vector
    // sort of overkill
    int i = 0;
    vector<PesMgr*>::iterator iv = pesmgrv.begin();
    for ( ;iv != pesmgrv.end(); ++i, ++iv ) 
    {
	if ( pesmgr == *iv ) 
	{
	    break;
	}
    }
    assert( iv != pesmgrv.end() );
    pesmgrv.erase(iv);
    CurMovie->removeItem(i);

    Zoom->setText("");
    updateMovieSlider(slider_begin);

    // necessary?
    // filename = "";
    
    ///////////////////////////////////////
    /// delete current parser!!
    //////////////////////////////////////

    // that's overkill - but safe for now!
    for ( unsigned i = clip_list.size(); i > 0; --i ) 
    {
	Clips->removeRow(i-1);
    }
    clip_list.clear();
    cut_list.clear();
    updateClipSum();
    
    delete pesmgr;
    pesmgr = 0;
    MovieScreen->setPesMgr(0);
    
    if ( pesmgrv.size() > 0 ) 
    {
	CurMovie->setCurrentItem(0);
	curMovieActivated(0);
    }
    else 
    {
	MovieScreen->repaint();
	Zoom->setText("");
	CurTime->setText("");
	CurGopB->setText("");
	CurGopE->setText("");
    }
}


void GopditQt::fileSave()
{
    if ( clip_list.size() == 0 ) 
    {
	error("No clips");
	return;
    }
    if ( filename.isEmpty() )
    {
	fileSaveAs();
	return;
    }
    
//    fprintf(stderr,"saving file %s\n",filename.ascii());
    
    // setup info for the progress bar
    
    off_t total_length = total_clip_length();

    total_length = (total_length) / prog_div + 1;

//    fprintf(stderr,"total_length %" OFF_T_FORMAT "\n", total_length);


#ifdef PROGRESS_DIALOG
    QProgressDialog progress("Saving Clips...", "Abort Saving", 
			     int(total_length), this,"progress",TRUE);

    prog_diag = &progress;
#endif
    string fname = string(filename.ascii());
    string ftype = string(filetype.ascii());
    if ( separate.getValue() ) 
    {
	fname = fname.substr(0,fname.rfind("."));
	saveFileSeparate(fname, ftype);
    }
    else 
    {
	saveFile(fname, ftype, clip_list);
    }

#ifdef PROGRESS_DIALOG
    progress.setProgress(int(total_length));
    prog_diag = 0;
#endif
    if ( erase_clips.getValue() ) 
    {
	// mark all clips
	for ( unsigned i = 0; i < clip_list.size(); ++i ) 
	{
	    Clips->selectRow(i);
	}
	editCut();
    }
    MovieScreen->repaint();
    modified = false;
}

void GopditQt::fileSaveAs()
{
    if ( clip_list.size() == 0 ) 
    {
	error("No clips");
	return;
    }
    processEvents();
    ParFileDialog filedialog(save_file, this, "Save As", TRUE);
    if ( filter.getValue() ) 
    {
	filedialog.setCaption(filedialog.caption() + " (Filtered)");
    }
    if ( separate.getValue() ) 
    {
	filedialog.setCaption(filedialog.caption() + " (Separate)");
    }

    // problem: if colons occur in the filename, the first part is
    // interpreted as a (url) protocol
    // I don't know how to avoid that interpretation
    if (pesmgr->getMovieName().find(":") == string::npos ) 
    {
	filedialog.setSelection(
	    QString(pesmgr->getMovieName().c_str()));
    }

    if ( filedialog.exec() == QDialog::Accepted )
    {
	processEvents();
	QString fn = filedialog.selectedExtendedFile();
	filename = fn;
	filetype = filedialog.selectedFilter();
	
	if ( access(filename,F_OK) == 0
	     && warning("File exists already")
	    ) 
	{
	    return;
	}
	
	fileSave();
    }
}

void GopditQt::fileExport()
{
    if ( clip_list.size() == 0 ) 
    {
	error("No clips");
	return;
    }

    processEvents();

    ParFileDialog filedialog(export_file, this, "Export As", TRUE);

    if (pesmgr->getMovieName().find(":") == string::npos ) 
    {
	filedialog.setSelection(
	    QString(pesmgr->getMovieName().c_str()));
    }
    
    if ( filedialog.exec() == QDialog::Accepted )
    {
	QString fn = filedialog.selectedExtendedFile();
	if ( access(fn,F_OK) == 0
	     && warning("File exists already")
	    ) 
	{
	    return;
	}

	refresh();
	processEvents();

	GopditEditListMgr(*this,*this).exportList(
	    string(fn.ascii()), 
	    string(filedialog.selectedFilter().ascii()),
	    clip_list);
    }
    // force repainting (gops were changed in between)
    refresh();
}

void GopditQt::fileImport()
{
    processEvents();
    ParFileDialog filedialog(import_file, this, "Import", TRUE,
			     QFileDialog::ExistingFile);
    if ( filedialog.exec() == QDialog::Accepted )
    {
	QString fn = filedialog.selectedFile();
	refresh();
	processEvents();

	importFile(string(fn.ascii()),
		   string(filedialog.selectedFilter().ascii()));
    }
    // force repainting (gops were changed in between)
    processEvents();
    refresh();
    processEvents();
}


void GopditQt::fileExit()
{
// probably not enough
    if ( modified 
	 && clip_list.size() > 0
	 && warning("The clip table was modified") )
    {
	return;
    }

    QValueList<int> width_list;
    width_list = Splitter->sizes();
    frame1_width.setValue(width_list.front());
    frame2_width.setValue(width_list.back());

    main_width.setValue(width());
    main_height.setValue(height());
    std::string rcfile = getenv("HOME");
    rcfile += "/.gopditrc";

    ParameterBase::writeResourceFile(rcfile);
    close();
}


void GopditQt::editCut()
{
    Mpeg2ClipList      tmplist;
    list<int>          selrows;

    cut_list.clear();

    for ( unsigned i = 0; i < clip_list.size(); ++i ) 
    {
	if ( Clips->isRowSelected(i) ) 
	{
	    selrows.push_front(i);
	    cut_list.push_back(clip_list[i]);
	}
	else 
	{
	    tmplist.push_back(clip_list[i]);
	}
    }
    for ( list<int>::iterator li = selrows.begin();
	  li != selrows.end(); ++li ) 
    {
	Clips->removeRow(*li);
    }

    clip_list = tmplist;

    updateClipSum();
}

void GopditQt::editCopy()
{
    cut_list.clear();

    for ( unsigned i = 0; i < clip_list.size(); ++i ) 
    {
	if ( Clips->isRowSelected(i) ) 
	{
	    cut_list.push_back(clip_list[i]);
	}
    }
}

void GopditQt::editPaste()
{
    for ( unsigned i = 0; i < cut_list.size(); ++i ) 
    {
	addClip(cut_list[i]);
    }
}


void GopditQt::curMovieActivated(int idx)
{
    assert(unsigned(idx) < pesmgrv.size());

    pesmgr = pesmgrv[idx];

    // also initialize first gop!
    slider_begin = 0;
    slider_end   = getFsize();
    MovieScreen->setPesMgr(pesmgr);
    CurGopB->setText("");
    Zoom->setText("");
    updateMovieSlider(getCurGop());
    refresh();
}


void GopditQt::filterToggle()
{
    filter.setValue(!filter.getValue());
}


void GopditQt::updateZoom()
{
    // compute zoom 
    off_t pos       = getCurGop();
    double p = double(getFsize()) / double(slider_end - slider_begin);
    if ( p == 1 ) 
    {
	Zoom->setText("");
    }
    else 
    {
	char buffer[256];
	sprintf(buffer,"%d %%",100 * int(p + 0.5));
	Zoom->setText(buffer);
    }

    updateMovieSlider(pos);

}

void GopditQt::zoomIn()
{
    zoomInSteps(1);
}


// must be renamed to zoomToggle
void GopditQt::zoomToggle()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }
    if ( slider_begin == 0
	&& slider_end == getFsize() ) 
    {
	zoomInSteps(toggle_steps.getValue());
    }
    else 
    {
	slider_begin = 0;
	slider_end   = getFsize();
	
	updateZoom();
    }
}

void GopditQt::zoomInSteps(int depth)
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    off_t pos       = getCurGop();
    off_t iv_length = (slider_end - slider_begin)/(2 << depth);
    off_t end_diff  = slider_end < pos + iv_length
	? pos + iv_length - slider_end
	: 0;
    off_t beg_diff  = (pos < slider_begin + iv_length) 
	? slider_begin + iv_length - pos
	: 0;

    slider_end      = pos + iv_length + beg_diff - end_diff;
    slider_begin    = pos + beg_diff - end_diff  - iv_length;

    updateZoom();

}


void GopditQt::zoomOut()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    off_t iv_length = (slider_end - slider_begin) / 2;
    off_t end_diff  = slider_end + iv_length > getFsize()
	? slider_end + iv_length - getFsize()
	: 0;
    off_t begin_diff = iv_length > slider_begin
	? iv_length - slider_begin
	: 0;

    slider_end      = min(getFsize(), slider_end + iv_length + begin_diff);
    slider_begin    = slider_begin < iv_length + end_diff
	? 0
	: slider_begin - iv_length - end_diff;

    updateZoom();
}


void GopditQt::movieSliderChanged(int)
{
    if ( pesmgr == 0 )
    {
	return;
    }

    // compute the relative position
    off_t  o = movieSliderOffset();

    if (! pesmgr->isRunning() ) 
    {
	pesmgr->setOffset(o);
	pesmgr->findNextGop();
	pesmgr->playOneSlice();
    }
    CurGopE -> setText(formatOff(getCurGop()).c_str());
}


void GopditQt::movieSliderReleased()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    stopPlaying();
    
    if ( !gop_inclusive.getValue() ) 
    {
	// compute the relative position
	off_t  o = movieSliderOffset();
	pesmgr->setOffset(o);
	pesmgr->findNextGop();
	pesmgr->play(1);
    }
}


void GopditQt::stopPlaying()
{
    MvPlay->setPixmap(play);
    pesmgr->setRunning(false);
}

void GopditQt::mvPlayClicked()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    if ( pesmgr->isRunning() ) 
    {
	stopPlaying();
    }
    else 
    {
	MvPlay->setPixmap(pause);
	pesmgr->play(-1);
	if ( gop_inclusive.getValue() ) 
	{
	    // not clean, we need a scheme to inhibit actions in between
	    pesmgr->playOneSlice(true);
	}
    }
}


void GopditQt::mvStopClicked()
{
    // how can that be reset?
    // we probably cannot change file buffers?
    // the interface must be changed!!
    if ( pesmgr == 0 ) 
    {
	return;
    }

    stopPlaying();
    pesmgr->setOffset(0);
}


void GopditQt::mvBwdClicked()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    stopPlaying();

    if ( gop_inclusive.getValue() ) 
    {
	pesmgr->findPrevGop(1);
	pesmgr->playOneSlice();
	updateMovieSlider(getCurGop());
    }
    else 
    {
	pesmgr->findPrevGop(2);
	pesmgr->play(1);
    }
}

void GopditQt::mvFwdClicked()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    stopPlaying();

    if ( !pesmgr->isValidCurGop() ) 
    {
	return;
    }

    if ( gop_inclusive.getValue() ) 
    {
	pesmgr->play(1,true);
	// interface isn't clean - dangerous as there might be actions here
	pesmgr->playOneSlice(true);
    }
    else 
    {
	pesmgr->play(1,true);
    }
}


/*
static void print_selections(const QTable& tab)
{
    for ( unsigned i = 0; i < unsigned(tab.numSelections()); ++i ) 
    {
	const QTableSelection& sel = tab.selection(i);
	fprintf(stderr, "sel %d: rb %d re %d cb %d ce %d \n", i,
		sel.bottomRow(), sel.topRow(), sel.leftCol(), sel.rightCol() );
    }
}
*/

void GopditQt::mvClipsDown()
{
    // search for clip(s)
    unsigned sel = clip_list.size();
    for ( unsigned i = 0; i < clip_list.size(); ++i ) 
    {
	if ( Clips->isRowSelected(i) ) 
	{
	    if ( sel < clip_list.size() ) 
	    {
		return;
	    }
	    else 
	    {
		sel = i;
	    }
	}
    }
    if ( clip_list.size() == 0 || sel >= clip_list.size() -1 ) 
    {
	return;
    }
    // swap
    swap(clip_list[sel],clip_list[sel+1]);
    Clips->swapRows(sel,sel+1);
    Clips->clearSelection();
    Clips->selectRow(sel+1);
    Clips->updateContents();
    Clips->updateHeaderStates();

}

void GopditQt::mvClipsUp()
{
    // search for exactly one selection
    unsigned sel = clip_list.size();
    for ( unsigned i = 0; i < clip_list.size(); ++i ) 
    {
	if ( Clips->isRowSelected(i) ) 
	{
	    if ( sel < clip_list.size() ) 
	    {
		return;
	    }
	    else 
	    {
		sel = i;
	    }
	}
    }
    if ( clip_list.size() == 0 || sel >= clip_list.size() || sel < 1 ) 
    {
	return;
    }
    // swap
    swap(clip_list[sel],clip_list[sel-1]);
    Clips->swapRows(sel,sel-1);
    Clips->clearSelection();
    Clips->selectRow(sel-1);
    Clips->updateContents();
    Clips->updateHeaderStates();

}

void GopditQt::clipsDoubleClicked(int row, int col, int button, const QPoint& mousePos)
{
    if ( col > 1 || clip_list[row].pesmgr != pesmgr ) 
    {
	return;
    }

    off_t pos = (col == 0)
	? clip_list[row].begin.off
	: clip_list[row].end.off;

    centerSlider(pos);
    updateMovieSlider(pos);
    refresh();
}


void GopditQt::updateClipSum()
{
    off_t clip_sum = total_clip_length();

    if ( clip_sum == 0 ) 
    {
	ClipSum->setText("");
    }
    else 
    {
	ClipSum->setText(formatOff(clip_sum).c_str());
    }
}

void GopditQt::clipBeginClicked()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    cur_clip.pesmgr    = pesmgr;
    cur_clip.begin.off = getCurGop();
    cur_clip.begin.ms  = getCurTime();

    report(string("new clip"));
    CurGopB->setText(formatOff(cur_clip.begin.off).c_str());
}

void GopditQt::clipEndClicked()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    // check that the pesmgr is still the same
    cur_clip.pesmgr  = pesmgr;
    cur_clip.end.off = getValidCurGop();
    cur_clip.end.ms  = getCurTime();
    
    if ( separate.getValue() ) 
    {
	ParFileDialog filedialog(save_file, this, "Save Clip As", TRUE);

	if (pesmgr->getMovieName().find(":") == string::npos ) 
	{
	    filedialog.setSelection(
		QString(pesmgr->getMovieName().c_str()));
	}
	if ( filedialog.exec() == QDialog::Accepted ) 
	{
	    processEvents();
	    cur_clip.filename = 
		string(filedialog.selectedExtendedFile().ascii());
	}
    }

    processEvents();

    if ( !addClip(cur_clip) ) 
    {
	return;
    }
    
    // update graphics as well!
    report(string("new clip"));

    CurGopB->setText("");
    cur_clip.begin    = ClipPoint();
    cur_clip.end      = ClipPoint();
    cur_clip.filename = string();
}


void GopditQt::clipCorrect()
{
//    fprintf(stderr,"clipCorrect()\n");
    // check, if there exactly one begin or end field selected
    if ( Clips->numSelections() != 1 )
    {
	error("No clip field selected");
	return;
    }
    const QTableSelection& sel = Clips->selection(0);
    int r = sel.bottomRow();
    int c = sel.leftCol();
    if ( r != sel.topRow()
	|| c != sel.rightCol()
	|| c > 1
	|| clip_list[r].pesmgr != pesmgr ) 
    {
	fprintf(stderr,"no clip selected\n");
	return;
    }

    Mpeg2Clip clip = clip_list[r];
    if ( c == 0 ) 
    {
	clip.begin.off = getCurGop();
	clip.begin.ms  = getCurTime();
    }
    else 
    {
	clip.end.off = getValidCurGop();
	clip.end.ms  = getCurTime();
    }
    if ( clip.end.off <= clip.begin.off ) 
    {
	error(string("clip end precedes clip begin\n"));
	return;
    }
    clip_list[r] = clip;
    updateClip(r,clip_list[r]);
    Clips->updateContents();
}

void GopditQt::updateClip(int row, const Mpeg2Clip& clip)
{
    Clips->setText(row,2,QString(clip.pesmgr->getMovieName().c_str()));
    if ( display_offset.getValue() ) 
    {
	Clips->setText(row,0,formatOff(clip.begin.off).c_str());
	Clips->setText(row,1,formatOff(clip.end.off).c_str());
    }
    else 
    {
	Clips->setText(row,0,formatTime(clip.begin.ms).c_str());
	Clips->setText(row,1,formatTime(clip.end.ms).c_str());
    }
    Clips->setText(row,3,formatOff(clip.end.off - clip.begin.off).c_str());

    if ( mark_clips.getValue() ) 
    {
	unsigned length = clip.end.ms - clip.begin.ms;
	if ( length < red_mark.getValue() ) 
	{
	    Clips->setPixmap(row,0,Red);
	}
	else if ( length < yellow_mark.getValue() ) 
	{
	    Clips->setPixmap(row,0,Yellow);
	}
	else 
	{
	    Clips->setPixmap(row,0,Green);
	}
    }

    for ( unsigned i = 0; i < 4; ++i ) 
    {
	Clips->adjustColumn(i);
    }

    updateClipSum();
    modified = true;
}

bool GopditQt::addClip(Mpeg2Clip& clip)
{
    if ( !Gopdit::addClip(clip) ) 
    {
	return false;
    }
    processEvents();
    int row = clip_list.size()-1;
    Clips->insertRows(row);
    processEvents();

    updateClip(row,clip);
    processEvents();

    return true;
}


void GopditQt::critical(string message)
{
    QMessageBox::critical(0,"gopdit",
			  QString(message.c_str()));
}


void GopditQt::error(string message)
{
    QMessageBox::warning(this,"gopdit",
			 QString(message.c_str()),
			 QMessageBox::Ok,
			 QMessageBox::NoButton,
			 QMessageBox::NoButton
	);
}


void* GopditQt::getMovieScreen()
{
    return (void*)MovieScreen->winId();
}


void GopditQt::processEvents()
{
    app.processEvents();
}

void GopditQt::report(string message)
{
    statusBar()->message(QString(message.c_str()));
}


void GopditQt::updateCurrentTime(unsigned t)
{
    CurTime->setText(formatTime(t).c_str());
}

bool GopditQt::updateProgressDialog(off_t c)
{
    if ( prog_diag != 0 ) 
    {
	prog_diag->setProgress(c/prog_div);
	processEvents();
	return !prog_diag->wasCanceled();
    }
    return true;
}

void GopditQt::refresh()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }

    processEvents();

    stopPlaying();

    movieSliderChanged(MovieSlider->value());
}

void GopditQt::updateMovieSlider(off_t c)
{
    if ( c == off_t(-1) ) 
    {
	return;
    }

    // shift, if out of range
    if ( c > slider_end ) 
    {
	slider_begin += c - slider_end;
	slider_end    = c;
    }
    else if ( c < slider_begin ) 
    {
	slider_end  -= slider_begin - c;
	slider_begin = c;
    }

    double p = double(c - slider_begin) / double(slider_end - slider_begin);
    int    v = int(p * MovieSlider->maxValue());

    if ( v != MovieSlider->value() ) 
    {
	MovieSlider->setValue(v);
    }
    CurGopE -> setText(formatOff(c).c_str());
    processEvents();
}


// returns true, when cancel is selected
bool GopditQt::warning(string message)
{
    return QMessageBox::warning(this,"gopdit",
				QString(message.c_str()),
				QMessageBox::Ignore,
				QMessageBox::Cancel,
				QMessageBox::NoButton
	) == QMessageBox::Cancel;
}


off_t GopditQt::getFsize() const
{
    return (pesmgr ==  0) ? 0 : pesmgr->getFsize();
}

off_t GopditQt::getCurGop() const
{
    return (pesmgr ==  0) ? 0 : pesmgr->getCurGop();
}

off_t GopditQt::getValidCurGop() const
{
    return (pesmgr ==  0) ? 0 : pesmgr->getValidCurGop();
}

unsigned GopditQt::getCurTime() const
{
    return (pesmgr ==  0) ? 0 : pesmgr->getCurTime();
}

string GopditQt::formatTime(unsigned ms) const
{
    char buffer[128];
    int secs = activate_time_scale.getValue()
	? int(ms * time_scale.getValue() / 1000)
	: int(ms / 1000);
    int s = secs % 60;
    secs = secs / 60;
    int m = secs % 60;
    int h = secs / 60;

    sprintf(buffer,"%d:%02d:%02d", h, m, s);
    return string(buffer);
}

string GopditQt::formatOff(off_t off) const
{
    char buffer[256];
    sprintf(buffer,"%" OFF_T_FORMAT ,off/(gop_div));
    return string(buffer);
}


void GopditQt::centerSlider(off_t pos)
{
    if ( slider_begin == 0 && slider_end == pesmgr->getFsize() ) 
    {
	return;
    }
    off_t sz  = slider_end - slider_begin;
    off_t end = pesmgr->getFsize();

    slider_begin = ( pos < (sz / 2)  ) ? 0 : pos = pos - sz / 2;
    slider_end   = ( pos > end - (sz / 2)) ? end : slider_begin + sz;
    slider_begin = slider_end - sz;
}

off_t GopditQt::movieSliderOffset()
{
    double p = double(MovieSlider->value())/double(MovieSlider->maxValue());
    return slider_begin + off_t(p * double(slider_end - slider_begin));
}

void GopditQt::helpAbout()
{
     QMessageBox::information( 
	this, "About...",
	"gopdit version " VERSION
	"\n\nCopyright (C) 2004-2006\n"
	"by Reinhard Enders <Reinhard.Enders@gmx.de>\n\n"
	"gopdit comes with ABSOLUTELY NO WARRENTY\n"
	"This is free software, for details visit\n"
	"http://www.gnu.org/copyleft/gpl.html",
	QMessageBox::NoButton);
}

void GopditQt::helpContents()
{
    if ( help == 0 ) 
    {
	QString home;
	home = QDir(DATADIR "/help.html").absPath();
	
	help = new HelpWindow(home, ".", 0, "help viewer");

	// help is distroyed, when closed, so we need to reset 'help' to 0
	QObject::connect( help, SIGNAL(destroyed()),
			  this, SLOT(helpDestroyed()) );

    }
    help->show();


}

void GopditQt::helpDestroyed()
{
    help = 0;
}

void GopditQt::preferences()
{
    if ( prefs == 0 ) 
    {
	prefs = new Preferences();
    }
    prefs->readParams();
    prefs->show();
}

void GopditQt::fileinfo()
{
    if ( pesmgr == 0 ) 
    {
	return;
    }
    if ( finfo == 0 ) 
    {
	finfo = new Fileinfo();
    }
    pesmgr->dumpInfo();

    finfo->showdata(*pesmgr);
    finfo->show();
}
