/*! 
  \file mpeg2output.cpp
  \brief

  \author R. Enders

  $Revision: 1.8 $
  $Id: mpeg2output.cpp,v 1.8 2006/01/06 12:17:59 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include mpeg2output.cpp
*/


#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "parameter.h"
#include "mpeg2output.h"

Parameter<string>   vo_driver("VoDriver", "", "preferred vo driver");
Parameter<bool>     nox("Nox",false,"when true, inhibits firing up X");

GopditOutput::GopditOutput()
    : output(0)
    , output_open(0)
{
    if ( !(output_open = findLibvoDriver(nox.getValue() 
					 ? "nullskip" 
					 : vo_driver.getValue().c_str()))  ) 
    {
	fprintf(stderr,"Can't find output driver\n");
	exit(1);
    }
    if ( !(output = output_open()) ) 
    {
	fprintf(stderr,"Can't open output driver\n");
    }
}

GopditOutput::~GopditOutput()
{
    if ( output != 0 && output->close != 0) 
    {
	output->close(output);
    }
    output = 0;
    output_open = 0;
}

vo_open_t * GopditOutput::findLibvoDriver(const char * desired_driver)
{
    int i;
    vo_driver_t const *drivers;

    drivers = vo_drivers();
    for (i = 0; drivers[i].name; i++)
    {
	if ( !strcmp("", desired_driver) 
	     || !strcmp(drivers[i].name, desired_driver) )
	{
	    fprintf(stderr,"opening vo driver %s\n", drivers[i].name);
	    return drivers[i].open;
	}
    }
    return NULL;
}

