#ifndef PESFILTER_H
#define PESFILTER_H

/*! 
  \file pesfilter.h
  \brief

  \author R. Enders

  $Revision: 1.7 $
  $Id: pesfilter.h,v 1.7 2006/01/06 12:18:00 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include pesfilter.h

*/

#include <inttypes.h>
#include <time.h>
#include <string>

#define PS_BUFFER_SIZE 4096

class GopditUi;

/*! 
  extracts mpeg2 video data from a ps file
 */
class PesFilter
{
public:
    PesFilter(GopditUi& pui,const char* pfilename);
    virtual ~PesFilter();
    off_t  getGopPh() const { return gop_ph; };
    int    seeko(off_t offset, int whence);
    // either invalid (off_t(-1)) or the offset of the last
    // package header before the current gop
    off_t  tello();
    off_t  getFsize() const { return fsize; }
    time_t getMtime() const { return mtime; }
    std::string    getFileName() const { return filename; }
    virtual bool   concat(off_t& sb, FILE* out, off_t from, off_t to,
			  unsigned& no_frames, float fps) = 0;
    virtual void   endOfProgram(FILE* out);
    virtual size_t read(uint8_t* ptr, size_t nmemb, bool search_only = false) = 0;
    virtual bool   searchGop();
protected:
    FILE*   file;
    std::string  filename;

    // file mtime
    time_t mtime;
    // file size
    off_t  fsize;
    // current file offset
    off_t  foffset;
    // file offset of previous pack header (-1) if not yet found
    off_t  prev_ph;
    // file offset of the pack header preceding a gop
    off_t  gop_ph;
    // state variables (all needed?)
    // fill size of buffer
    size_t bsize;
    // data to be skipped
    bool   skip;
    // currently in video mode
    bool   video;

    uint8_t buffer[4];
    size_t  bunloaded;

    bool   init_state();

    GopditUi&  ui;
};


#endif // PESFILTER_H
