/*! 
  \file preferences.cpp
  \brief preferences dialog

  \author R. Enders

  $Revision: 1.18 $
  $Id: preferences.cpp,v 1.18 2006/01/06 12:18:00 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include preferences.cpp
*/

#include "config.h"
#include <assert.h>
#include <math.h>

#include <qcheckbox.h>
#include <qlineedit.h>
#include <qdial.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qtooltip.h>

#include "parameter.h"
#include ".ui/preferencesbase.h"
#include "preferences.h"

using namespace std;

Preferences::Preferences(QWidget* parent, const char* name,
			 bool modal, WFlags fl)
    : PreferencesDialog(parent, name, modal, fl)
{}

Preferences::~Preferences()
{}

/*
  TimeScale n in integer ranges f  -50 .. 50
  as factor f it should be in the range between 0.1 and 10,
  so f = exp(n / 20)   with the inverse
     n = ln(f) * 20
*/


#ifdef EXPERIMENTAL
void Preferences::setToolTip(QWidget& q, ParameterBase& p)
{
    if ( p.getDescription() != ""
	&& QToolTip::textFor(&q) == QString::null ) 
    {
	QToolTip::add(&q, tr(p.getDescription().c_str()) );
    }
}

void Preferences::readEditLineParam(QWidget& q, ParameterBase& p)
{
    dynamic_cast<QLineEdit&>(q).setText(
	QString(dynamic_cast<Parameter<string>&>(p).getValue().c_str()));
    
    setToolTip(q,p);
}

void Preferences::readCheckBoxParam(QWidget& q, ParameterBase& p)
{
    dynamic_cast<QCheckBox&>(q).setChecked(
	dynamic_cast<Parameter<bool>&>(p).getValue());
    
    setToolTip(q,p);
}

void Preferences::readQSliderParam(QWidget& q, ParameterBase& p)
{
    readWidgetParam<QSlider,Parameter<int> >(q,p);
}

template <class Q, class P> void Preferences::readWidgetParam(
    QWidget& q, ParameterBase& p)
{
    dynamic_cast<Q&>(q).setValue(
	dynamic_cast<P&>(p).getValue());
    
    setToolTip(q,p);
}
#endif // EXPERIMENTAL

void Preferences::readParams()
{
    ParameterMap parmap = ParameterBase::getParameterMap();
    for ( ParameterMap::iterator mi = parmap.begin();
	  mi != parmap.end(); ++mi ) 
    {
//	cerr << "parameter " << mi->first << endl;
	if ( mi->first == "FilterCommand" ) 
	{
#ifdef EXPERIMENTAL
	    readEditLineParam(*FilterCommand, *(mi->second));

#else
	    FilterCommand->setText(
		QString(dynamic_cast<Parameter<string>*>(mi->second)
			->getValue().c_str()));
#endif
	}
	else if ( mi->first == "ActivateFilter" ) 
	{
	    ActivateFilter->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "ActivateTimeScale" ) 
	{
	    ActivateTimeScale->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "AdjustGopTiming" ) 
	{
	    AdjustGopTiming->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "DisplayCommercials" ) 
	{
	    DisplayCommercials->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "DisplayOffset" ) 
	{
	    DisplayOffset->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "EditPointAdj" ) 
	{
	    EditPointAdj->setValue(
		dynamic_cast<Parameter<int>*>(mi->second)->getValue());
	}
	else if ( mi->first == "GopInclusive" ) 
	{
	    GopInclusive->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "IFrames" ) 
	{
	    IFrames->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "ImportMpeg" ) 
	{
	    ImportMpeg->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "MarkClips" ) 
	{
	    MarkClips->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "SaveFileFilters" ) 
	{
	    MpegExtension->setText(
		QString(dynamic_cast<Parameter<string>*>(mi->second)
			->getValue().c_str()));
	}
	else if ( mi->first == "ExportFileFilters" ) 
	{
	    EditListExtension->setText(
		QString(dynamic_cast<Parameter<string>*>(mi->second)
			->getValue().c_str()));
	}
	else if ( mi->first == "Separate" ) 
	{
	    Separate->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "TimeScale" ) 
	{
	    TimeScale->setValue(
		int(20 * log(dynamic_cast<Parameter<float>*>(mi->second)
			     ->getValue())));
	}
	else if ( mi->first == "ToggleZoomSteps" ) 
	{
	    ToggleZoomSteps->setValue(
		dynamic_cast<Parameter<int>*>(mi->second)->getValue());
	}
	else if ( mi->first == "Touch" ) 
	{
	    Touch->setChecked(
		dynamic_cast<Parameter<bool>*>(mi->second)->getValue());
	}
	else if ( mi->first == "RedClipMark" ) 
	{
	    RedClipMark->setValue(
		dynamic_cast<Parameter<unsigned>*>(mi->second)->getValue()
		/ 1000 );
	}
	else if ( mi->first == "SuppressClip" ) 
	{
	    SuppressClip->setValue(
		dynamic_cast<Parameter<unsigned>*>(mi->second)->getValue()
		/ 1000 );
	}
	else if ( mi->first == "YellowClipMark" ) 
	{
	    YellowClipMark->setValue(
		dynamic_cast<Parameter<unsigned>*>(mi->second)->getValue()
		/ 1000 );
	}
    }
}

void Preferences::writeParams()
{
    ParameterMap parmap = ParameterBase::getParameterMap();
    for ( ParameterMap::iterator mi = parmap.begin();
	  mi != parmap.end(); ++mi ) 
    {
	if ( mi->first == "FilterCommand" ) 
	{
	    dynamic_cast<Parameter<string>*>(mi->second)
		->setValue(string(FilterCommand->text().ascii()));
	}
	else if ( mi->first == "ActivateFilter" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(ActivateFilter->isChecked());
	}
	else if ( mi->first == "ActivateTimeScale" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(ActivateTimeScale->isChecked());
	}
	else if ( mi->first == "AdjustGopTiming" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(AdjustGopTiming->isChecked());
	}
	else if ( mi->first == "DisplayCommercials" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(DisplayCommercials->isChecked());
	}
	else if ( mi->first == "DisplayOffset" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(DisplayOffset->isChecked());
	}
	else if ( mi->first == "EditPointAdj" ) 
	{
	    dynamic_cast<Parameter<int>*>(mi->second)
		->setValue(EditPointAdj->value());
	}
	else if ( mi->first == "GopInclusive" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(GopInclusive->isChecked());
	    
	}
	else if ( mi->first == "IFrames" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(IFrames->isChecked());
	}
	else if ( mi->first == "ImportMpeg" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(ImportMpeg->isChecked());
	}
	else if ( mi->first == "MarkClips" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(MarkClips->isChecked());
	}
	else if ( mi->first == "SaveFileFilters" ) 
	{
	    dynamic_cast<Parameter<string>*>(mi->second)
		->setValue(string(MpegExtension->text().ascii()));
	}
	else if ( mi->first == "ExportFileFilters" ) 
	{
	    dynamic_cast<Parameter<string>*>(mi->second)
		->setValue(string(EditListExtension->text().ascii()));
	}
	else if ( mi->first == "Separate" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(Separate->isChecked());
	}
	else if ( mi->first == "TimeScale" ) 
	{
	    dynamic_cast<Parameter<float>*>(mi->second)
		->setValue(exp(TimeScale->value() / 20.0));
	}
	else if ( mi->first == "ToggleZoomSteps" ) 
	{
	    dynamic_cast<Parameter<int>*>(mi->second)
		->setValue(ToggleZoomSteps->value());
	}
	else if ( mi->first == "Touch" ) 
	{
	    dynamic_cast<Parameter<bool>*>(mi->second)
		->setValue(Touch->isChecked());
	}
	else if ( mi->first == "RedClipMark" ) 
	{
	    dynamic_cast<Parameter<unsigned>*>(mi->second)
		->setValue(RedClipMark->value() * 1000);
	}
	else if ( mi->first == "SuppressClip" ) 
	{
	    dynamic_cast<Parameter<unsigned>*>(mi->second)
		->setValue(SuppressClip->value() * 1000);
	}
	else if ( mi->first == "YellowClipMark" ) 
	{
	    dynamic_cast<Parameter<unsigned>*>(mi->second)
		->setValue(YellowClipMark->value() * 1000);
	}
    }
}

void Preferences::preferencesButtonOk()
{
    writeParams();
    std::string rcfile = getenv("HOME");
    rcfile += "/.gopditrc";

    ParameterBase::writeResourceFile(rcfile);
}
