/*! 
  \file psmgr.cpp
  \brief

  filter that extracts PES dynamically from MPEG2-PS

  \author R. Enders

  $Revision: 1.19 $
  $Id: psmgr.cpp,v 1.19 2006/01/06 12:18:00 reinhard Exp $
  
  Copyright (C) 2004-2006 by Reinhard Enders
  Reinhard.Enders@gmx.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  http://www.gnu.org/copyleft/gpl.html

  \include psmgr.cpp
*/

#include "config.h"
#include "parameter.h"
#include "psmgr.h"
#include "gopditui.h"

#undef FSEEK
#undef FTELL
#ifdef HAVE_FSEEKO
# define FSEEK      fseeko
# define FTELL      ftello
#else
# define FSEEK      fseek
# define FTELL      ftell
#endif



#include <assert.h>

Parameter<bool> adjust_gop_timing("AdjustGopTiming",false);


// first version 0: skip, 1: keep, 2: audio, 3: video
//               4: pack start 5: gop start 6: picture start
uint8_t ps_action[] =
{
    /* 00 - 0f */
    6, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 10 - 1f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 20 - 2f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 30 - 3f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 40 - 4f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 50 - 5f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 60 - 6f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 70 - 7f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 80 - 8f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* 90 - 9f */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* a0 - af */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    /* b0 - bf */
    0, 0, 0, 1, 1, 1, 0, 1, 5, 1, 4, 0, 0, 0, 0, 0,
    /* c0 - cf */
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    /* d0 - df */
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    /* e0 - ef */
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
    /* f0 - ff */
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
};


// mode search_only is a real quick ugly hack!!!!
size_t PsMgr::read(uint8_t* ptr, size_t nmemb, bool search_only)
{
    // absolutely primitive version
    // search for PES headers and print out the byte code
    // compare muxed and demuxed files

    // absolutely unoptimized
    // improvement: use buffer(s)


    int     in;
    int     hdl_cd = -1;    // search for header data length
    int     pd_cd  = -1;    // search for packet data
    size_t  bsize  = 0;     // so far output

    while ( ( bsize < nmemb  || hdl_cd >= 0 || pd_cd >= 0 ) 
	    && (in = getc(file)) != EOF )
    {

	assert( !(hdl_cd >= 0 || pd_cd >= 0) || skip );

	if ( !skip && !search_only ) 
	{
	    *ptr = buffer[0];
	    ++bsize;
	    ++ptr;
	}

	// not efficient!!!
	buffer[0] = buffer[1];
	buffer[1] = buffer[2];
	buffer[2] = buffer[3];
	buffer[3] = in;

	// not efficient (use a small fsm)
	if ( buffer[2] == 1 && buffer[0] == 0 && buffer[1] == 0
	     // seems to be necessary, though I'm not sure, if
	     // mpeg streams are correct, if this is needed
	    && pd_cd < 0 && hdl_cd < 0 ) 
	{
	    switch ( ps_action[buffer[3]] ) 
	    {
	    case 0:
		skip = true;
		break;
	    case 1:
	    case 6:
		skip = !video;
		break;
	    case 2:
		skip = true;
		video = false;
		break;
	    case 3:
		skip = true;
		video = true;
		// search for field header data length
		hdl_cd = 5;
		break;
	    case 4:
		skip = true;
		// if that's too expensive, we could keep the offset
		prev_ph = FTELL(file) - 4;
		break;
	    case 5:
		// similar to 1
		// gop marker found
		skip = !video;
		if ( !skip ) 
		{
		    gop_ph = prev_ph;
		    if ( search_only && prev_ph != off_t(-1) ) 
		    {
			return 1;
		    }
		}
		break;
	    default:
		skip = true;
		video = false;
		break;
	    }
	}
	else if ( hdl_cd == 0 ) 
	{
	    pd_cd = in + 4;
	}
	else if ( pd_cd == 0 ) 
	{
	    skip  = !video;
	}
	--hdl_cd;
	--pd_cd;
    }

    while ( !search_only && !skip && bsize < nmemb && bunloaded < 4 ) 
    {
	*ptr = buffer[bunloaded];
	++bunloaded;
	++bsize;
	++ptr;
    }

#ifdef GOPDIT_TRACING
    if ( hdl_cd >= 0 || pd_cd >= 0 ) 
    {
	fprintf(stderr,"incorrect length in filter at %llx\n",
		FTELL(file));
    }
#endif // GOPDIT_TRACING

    assert(hdl_cd < 0);
    assert(pd_cd < 0);

    return bsize;
}

#define NEW_VERSION
#ifdef NEW_VERSION
// it's only approximative - the scan is too rough, so too many picture
// starts are detected
bool   PsMgr::concat(off_t& sb, FILE* out, off_t from, off_t to,
		     unsigned& no_frames, float fps)
{
    FSEEK(file,from,SEEK_SET);

    // used for marking gop as broken
    bool start = true;

    // true, if inside a video stream (needs refinement)
    bool invideo = false;

    int  counter = 0;

    const size_t fbuffer_size = 4096;
    size_t  size;
    // spare room for gop data
    uint8_t fbuffer[fbuffer_size + 4];
    uint8_t buffer[4];

    for ( unsigned i = 0; i < 4; ++i ) 
    {
	buffer[i] = 0xff;
    }

    int overlap = 0;

    while ( from < to ) 
    {
	size_t p = 0;
	size_t t;
	size = from + fbuffer_size <= to ? fbuffer_size : to - from;
	size = fread(fbuffer,sizeof(uint8_t),size,file);
	from += size;
	sb += size;

	// avoid overlapping control sequences (in the case of picture start)

	while ( p < size ) 
	{
	    buffer[0] = buffer[1];
	    buffer[1] = buffer[2];
	    buffer[2] = buffer[3];
	    buffer[3] = fbuffer[p];
	    ++p;

	    // not efficient (use a small fsm)
	    if ( buffer[2] == 1 && buffer[0] == 0 && buffer[1] == 0
		&& overlap <= 0 )
	    {
		overlap = 4;
		switch ( ps_action[buffer[3]] ) 
		{
		case 2:
		    invideo = false;
		    break;
		case 3:
		    invideo = true;
		    break;
		case 4:
		    invideo = false;
		    break;
		case 6: // picture start
		    if ( invideo ) 
		    {
			++no_frames;
		    }
		    else 
		    {
//			fprintf(stderr,"wrong picture excluded\n");
		    }
		    break;
		case 5: // gop start
		    if ( !invideo ) 
		    {
//			fprintf(stderr,"wrong gop excluded\n");
			break;
		    }
		    if ( p + 4 > size ) // extend for gop data
		    {
			t = fread(fbuffer+size,sizeof(uint8_t),4,file);
			assert( t == 4 );
			from += 4;
			size += 4;
			sb += 4;
		    }
		    gopModify(fbuffer+p,no_frames,fps,start);
		    if ( start ) 
		    {
			start = false;
		    }
		    break;
		}
	    }
	    --overlap;
	}


	if ( size != fwrite(fbuffer,sizeof(uint8_t),size,out) 
	     || ferror(out) != 0 ) 
	{
	    return false;
	}

	counter = (counter +1) % 256;
	if ( counter == 0 ) 
	{
	   if ( !ui.updateProgressDialog(sb) ) 
	   {
	       return false;
	   } 
	}
    }
    return true;
}

    // GOP header variables
    // |    0    |    1    |    2   |    3    |
    //  7 65432 107654 3 210765 432107 6 543210
    //  1 11111 111111 1 111111 111111 1 1
    //  d hour  min    m sec    pic    c b
    //  r              a               l roken
    //  o              r               osed
    //  p              k


void PsMgr::gopModify(uint8_t* ptr, unsigned no_frames, float fps, bool broken)
{

    uint8_t  cls = (ptr[3] & 0x40);

    if ( ! adjust_gop_timing.getValue() ) 
    {
	// just add a broken flag
	if ( !cls && broken ) 
	{
	    ptr[3] |= 0x20;
	}
    }
    else 
    {
	uint8_t  drop = (ptr[0] & 0x80);
	uint8_t  hour;
	uint8_t  min;
	uint8_t  sec;
	uint8_t  pic;
	uint8_t  brk = (ptr[3] & 0x20);
	
	brk = (brk || (!cls && broken)) ? 0x20 : 0;
	
	unsigned tm = unsigned(no_frames / fps);
	pic = no_frames - unsigned(tm * fps);
	sec = tm % 60;
	tm  = tm / 60;
	min = tm % 60;
	hour = tm / 60;
	
#ifdef GOPDIT_TRACING
	fprintf(stderr, "GOP %04d  %02d:%02d:%02d.%02d d:%d c:%d b:%d\n",
		no_frames, hour, min, sec, pic, drop > 0, cls > 0, brk > 0);
#endif // GOPDIT_TRACING
	
	ptr[0] = drop | (hour << 2) | (min >> 4) ;
	ptr[1] = (min << 4) | (ptr[1] & 0x08) | (sec >> 3);
	ptr[2] = (sec << 5) | (pic >> 1);
	ptr[3] = (pic << 7) | cls | brk | (ptr[3] & 0x1f);
    }
}

#else // NEW_VERSION

bool   PsMgr::concat(off_t& sb, FILE* out, off_t from, off_t to,
		     unsigned& no_frames, float fps)
{
    FSEEK(file,from,SEEK_SET);

    bool start = true;

    int  counter = 0;

    const size_t buffer_size = 4096;
    size_t  size;
    uint8_t buffer[buffer_size];

    while ( from < to ) 
    {
	size = from + buffer_size <= to ? buffer_size : to - from;
	size = fread(buffer,sizeof(uint8_t),size,file);
	size = fwrite(buffer,sizeof(uint8_t),size,out);
	sb += size;
	from += size;
	if ( start == true ) 
	{
	    start = false;
	    // check it's a header start
	    assert( buffer[0] == 0
		    && buffer[1] == 0
		    && buffer[2] == 0x01
		    && buffer[3] == 0xba );
	}

	counter = (counter +1) % 256;
	if ( counter == 0 ) 
	{
	   if ( !ui.updateProgressDialog(sb) ) 
	   {
	       return false;
	   } 
	}
    }
    return true;
}

#endif // NEW_VERSION
